/******** <lalVerbatim file="BlockBuildOutputCV"> ********
Author: Tibbits, M. M.
$Id: BlockBuildOutput.c,v 1.25 2003/07/06 20:06:08 jolien Exp $
********* </lalVerbatim> ********/

/******************************* <lalLaTeX file="BlockBuildOutputC">
\subsection{Module \texttt{BlockBuildOutput.c}}
\label{s:BlockBuildOutput.c}

Routine that builds the output from the result vector stored in the
parameter structure for insertion into the metadata database and frames.

\subsubsection*{Prototypes}
\input{BlockBuildOutputCP}
\index{\texttt{BlockBuildOutput()}}

\subsection*{Description}

Routine that builds the output from the result vector stored in the
parameter structure for insertion into the metadata database and frames.


\subsection*{Tables Used}

\begin{itemize}
\item \texttt{search::summary}
\item \texttt{summ::value}
\end{itemize}

\subsection*{Uses}
 
\begin{itemize}
\item \texttt{LALMalloc}
\item \texttt{LALCalloc}
\end{itemize}

\subsection*{Notes}

\vfill{\footnotesize\input{BlockBuildOutputCV}}

*************************************************** </lalLaTeX> */

#include <math.h>
#include "Block.h"

/* define BLOCKSEARCH__DATABASE_DEBUG */

NRCSID( BLOCKBUILDOUTPUTC, "$Id: BlockBuildOutput.c,v 1.25 2003/07/06 20:06:08 jolien Exp $");

/************************ <lalVerbatim file="BlockBuildOutputCP"> */
void BuildOutput
(
	LALStatus		*status,
	LALSearchOutput		*output,
	BlockSearchParams	*params
)
/*********************************************** </lalVerbatim> */
{
	dataBase	*dbEntry	= NULL;
	outPut		*outputPtr	= NULL;
const	CHAR		*PROGRAM	= "Block Normal Search";

	INITSTATUS( status, "BlockBuildOutput", BLOCKBUILDOUTPUTC );

	output->numOutput = 1;

	/*  Allocate Memory  */
	output->result = outputPtr = LALMalloc( (output->numOutput) * sizeof(outPut) );
	if ( !output->result )
	{
		ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
	}

	outputPtr[0].indexNumber	= 0;
	outputPtr[0].search		= experimental;
	outputPtr[0].significant	= 1;	/* 1 - IS significant */
	outputPtr[0].states		= 0;
	outputPtr[0].optional		= 0;

	/*
	 * Start a linked list of database entries - the head of the list
	 * starts with the "results" field
	 */

	outputPtr[0].results = dbEntry = LALCalloc(1, sizeof(dataBase));
	if ( !dbEntry )
	{
		ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
	}

	dbEntry->previous = NULL;



	/* Shared Object Name */
	strncpy( dbEntry->tableName, "search_summary", dbNameLimit );
	strncpy( dbEntry->columnName, "shared_object", dbNameLimit );

	dbEntry->type		= char_s_ptr;
	dbEntry->numberRows	= 1;

	dbEntry->rows.chars = LALMalloc(sizeof(CHAR)*(strlen(PROGRAM) + 1) );
	strcpy(dbEntry->rows.chars, PROGRAM);

	dbEntry->rowDimensions = LALMalloc(sizeof(UINT4));

	/* Don't include terminating null */
	dbEntry->rowDimensions[0] = strlen(PROGRAM);

	dbEntry->next = LALCalloc(1, sizeof(dataBase));
	dbEntry->next->previous = dbEntry;
	dbEntry = dbEntry->next;

        if ( !dbEntry )
	{
                ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
	}


	/* LAL CVS tag */
	strncpy( dbEntry->tableName, "search_summary", dbNameLimit );
	strncpy( dbEntry->columnName, "lal_cvs_tag", dbNameLimit );

	dbEntry->type		= char_s_ptr;
	dbEntry->numberRows	= 1;

	dbEntry->rows.chars = (CHAR *) LALMalloc( (strlen(LAL_CVS_TAG)+1) * sizeof(CHAR) );
	if ( !dbEntry->rows.chars )
	{
		ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
	}

	strcpy( dbEntry->rows.chars, LAL_CVS_TAG);

	dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
	if ( !dbEntry->rowDimensions )
	{
		ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
	}

	*(dbEntry->rowDimensions) = strlen(LAL_CVS_TAG);

        dbEntry->next = LALCalloc(1, sizeof(dataBase));
        dbEntry->next->previous = dbEntry;
        dbEntry = dbEntry->next;

        if ( !dbEntry )
	{
		ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
	}


	/* LALWrapper CVS tag */
	strncpy( dbEntry->tableName, "search_summary", dbNameLimit );
	strncpy( dbEntry->columnName, "lalwrapper_cvs_tag", dbNameLimit );

	dbEntry->type		= char_s_ptr;
	dbEntry->numberRows	= 1;

	dbEntry->rows.chars = (CHAR *) LALMalloc( (strlen(LAL_WRAPPER_CVS_TAG)+1) * sizeof(CHAR) );
	if ( !dbEntry->rows.chars )
	{
		ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
	}

	strcpy( dbEntry->rows.chars, LAL_WRAPPER_CVS_TAG);

	dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
	if ( !dbEntry->rowDimensions )
	{
		ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
	}

	*(dbEntry->rowDimensions) = strlen(LAL_WRAPPER_CVS_TAG);

        dbEntry->next = LALCalloc(1, sizeof(dataBase));
        dbEntry->next->previous = dbEntry;
        dbEntry = dbEntry->next;

        if ( !dbEntry )
	{
                ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
	}


	/* In Start time */
	strncpy( dbEntry->tableName, "search_summary", dbNameLimit);
	strncpy( dbEntry->columnName, "in_start_time", dbNameLimit );

	dbEntry->type       = int_4s;
	dbEntry->numberRows = 1;
	dbEntry->rows.int4s = LALMalloc( sizeof(INT4) );
        if ( !dbEntry->rows.int4s )
        {
                ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
        }

	*(dbEntry->rows.int4s) = params->times.startSec;
	dbEntry->rowDimensions = NULL;

	dbEntry->next = LALCalloc(1, sizeof(dataBase));
	dbEntry->next->previous = dbEntry;
	dbEntry = dbEntry->next;
        if ( !dbEntry )
        {                
                ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
        }


	strncpy( dbEntry->tableName, "search_summary", dbNameLimit );
	strncpy( dbEntry->columnName, "in_start_time_ns", dbNameLimit );

	dbEntry->type       = int_4s;
	dbEntry->numberRows = 1;
	dbEntry->rows.int4s = LALMalloc( sizeof(INT4) );

	*(dbEntry->rows.int4s) = params->times.startNan;
	dbEntry->rowDimensions = NULL;
        if ( !dbEntry->rows.int4s )
        {
                ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
        }

	dbEntry->next = LALCalloc(1, sizeof(dataBase));
	dbEntry->next->previous = dbEntry;
	dbEntry = dbEntry->next;
	if ( !dbEntry )
	{
		ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
	}



	/*  In End time  */
	strncpy( dbEntry->tableName, "search_summary", dbNameLimit );
	strncpy( dbEntry->columnName, "in_end_time", dbNameLimit );

	dbEntry->type       = int_4s;
	dbEntry->numberRows = 1;
	dbEntry->rows.int4s = LALMalloc( sizeof(INT4) );
        if ( !dbEntry->rows.int4s )
        {
                ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
        }

	*(dbEntry->rows.int4s) = params->times.stopSec;
	dbEntry->rowDimensions = NULL;

	dbEntry->next = LALCalloc(1, sizeof(dataBase));
	dbEntry->next->previous = dbEntry;
	dbEntry = dbEntry->next;
        if ( !dbEntry )
        {
                ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
        }

	strncpy( dbEntry->tableName, "search_summary", dbNameLimit );
	strncpy( dbEntry->columnName, "in_end_time_ns", dbNameLimit );

	dbEntry->type       = int_4s;
	dbEntry->numberRows = 1;
	dbEntry->rows.int4s = LALMalloc( sizeof(INT4) );
        if ( !dbEntry->rows.int4s )
        {
                ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
        }


	*(dbEntry->rows.int4s) = params->times.stopNan;
	dbEntry->rowDimensions = NULL;

	dbEntry->next = LALCalloc(1, sizeof(dataBase));
	dbEntry->next->previous = dbEntry;
	dbEntry = dbEntry->next;
        if ( !dbEntry )
        {
                ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
        }



        /* Out Start time */
        strncpy( dbEntry->tableName, "search_summary", dbNameLimit);
        strncpy( dbEntry->columnName, "out_start_time", dbNameLimit );

        dbEntry->type       = int_4s;
        dbEntry->numberRows = 1;
        dbEntry->rows.int4s = LALMalloc( sizeof(INT4) );
        if ( !dbEntry->rows.int4s )
        {
                ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
        }

        *(dbEntry->rows.int4s) = params->times.startSec;
        dbEntry->rowDimensions = NULL;

        dbEntry->next = LALCalloc(1, sizeof(dataBase));
        dbEntry->next->previous = dbEntry;
        dbEntry = dbEntry->next;
        if ( !dbEntry )
        {
                ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
        }

        strncpy( dbEntry->tableName, "search_summary", dbNameLimit );
        strncpy( dbEntry->columnName, "out_start_time_ns", dbNameLimit );

        dbEntry->type       = int_4s;
        dbEntry->numberRows = 1;
        dbEntry->rows.int4s = LALMalloc( sizeof(INT4) );
        if ( !dbEntry->rows.int4s )
        {
                ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
        }

        *(dbEntry->rows.int4s) = params->times.startNan;
        dbEntry->rowDimensions = NULL;

        dbEntry->next = LALCalloc(1, sizeof(dataBase));
        dbEntry->next->previous = dbEntry;
        dbEntry = dbEntry->next;
        if ( !dbEntry )
        {
                ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
        }



        /* Out End time */
        strncpy( dbEntry->tableName, "search_summary", dbNameLimit );
        strncpy( dbEntry->columnName, "out_end_time", dbNameLimit );

        dbEntry->type       = int_4s;
        dbEntry->numberRows = 1;
        dbEntry->rows.int4s = LALMalloc( sizeof(INT4) );
        if ( !dbEntry->rows.int4s )
        {
                ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
        }

        *(dbEntry->rows.int4s) = params->times.stopSec;
        dbEntry->rowDimensions = NULL;

        dbEntry->next = LALCalloc(1, sizeof(dataBase));
        dbEntry->next->previous = dbEntry;
        dbEntry = dbEntry->next;
        if ( !dbEntry )
        {
                ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
        }

        strncpy( dbEntry->tableName, "search_summary", dbNameLimit );
        strncpy( dbEntry->columnName, "out_end_time_ns", dbNameLimit );

        dbEntry->type       = int_4s;
        dbEntry->numberRows = 1;
        dbEntry->rows.int4s = LALMalloc( sizeof(INT4) );
        if ( !dbEntry->rows.int4s )
        {
                ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
        }

        *(dbEntry->rows.int4s) = params->times.stopNan;
        dbEntry->rowDimensions = NULL;

        /* Terminate the list */
        dbEntry->next = NULL;

	RETURN(status);
}


/************************ <lalVerbatim file="BlockBuildOutputCP"> */
void AddResults
(
	LALStatus		*status,
	LALSearchOutput		*output,
	BlockSearchParams	*params
)
/*********************************************** </lalVerbatim> */
{
  BlockResult	*tempResult= NULL;
  BlockResult	*current   = NULL;
  dataBase	*dbEntry   = NULL;
  outPut	*outputPtr = NULL;
  REAL8		dataStart  = 0.0;
  REAL8		timeZero   = 0.0;
  INT4		stringLength;
  INT4		timeNan    = 0;
  INT4		timeSec    = 0;
  INT4		index;
  CHAR		tmp[50];

  INITSTATUS( status, "AddResults", BLOCKBUILDOUTPUTC );

  if( !params->results)
    {
      RETURN( status );
    }

  dataStart = params->times.startSec + ( ((REAL8)(params->times.startNan)) / 1000000000.0 );

  if ( params->resultsToStore )
    {
      params->lastResultStored->next = dbEntry = LALCalloc(1, sizeof(dataBase));
      dbEntry->previous = params->lastResultStored;
    }
  else
    {
      params->resultsToStore = dbEntry = LALCalloc(1, sizeof(dataBase));
      params->resultsToStore->previous = NULL;
    }

  if ( !dbEntry )
    {
      ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
    }


  /*  Search Tag  */
  strncpy( dbEntry->tableName, "sngl_block", dbNameLimit );
  strncpy( dbEntry->columnName, "search", dbNameLimit );

  dbEntry->type           = char_s_ptr;
  dbEntry->numberRows     = params->numberResults;

  dbEntry->rows.chars = (CHAR *) LALMalloc( (strlen("Block Search")+1) * sizeof(CHAR) *params->numberResults);
  if ( !dbEntry->rows.chars )
    {
      ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
    }

  stringLength = strlen("Block Search");
  dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) * params->numberResults );
  if ( !dbEntry->rowDimensions )
    {
      ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
    }

  for (index = 0, current = params->results; (index < params->numberResults); index++, current = ((BlockResult*)(current->next)))
    {
      *(dbEntry->rowDimensions + index) = stringLength;
      strncpy( dbEntry->rows.chars + index * stringLength, "Block Search", stringLength * sizeof(CHAR) );
    }

  dbEntry->next = LALCalloc(1, sizeof(dataBase));
  if ( !dbEntry->next )
    {
      ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
    }

  dbEntry->next->previous = dbEntry;
  dbEntry = dbEntry->next;



  /*  IFO Tag  */
  strncpy( dbEntry->tableName, "sngl_block", dbNameLimit );
  strncpy( dbEntry->columnName, "ifo", dbNameLimit );

  dbEntry->type           = char_s_ptr;
  dbEntry->numberRows     = params->numberResults;

  dbEntry->rows.chars = (CHAR *) LALMalloc( (strlen(params->ifo)+1) * sizeof(CHAR) *params->numberResults);
  if ( !dbEntry->rows.chars )
    {
      ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
    }

  stringLength = strlen(params->ifo);
  dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) * params->numberResults );
  if ( !dbEntry->rowDimensions )
    {
      ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
    }

  for (index = 0, current = params->results; (index < params->numberResults); index++, current = ((BlockResult*)(current->next)))
    {
      *(dbEntry->rowDimensions + index) = stringLength;
      strncpy( dbEntry->rows.chars + index * stringLength, params->ifo, stringLength * sizeof(CHAR) );
    }

  dbEntry->next = LALCalloc(1, sizeof(dataBase));
  if ( !dbEntry->next )
    {
      ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
    }

  dbEntry->next->previous = dbEntry;
  dbEntry = dbEntry->next;



  /*  Channel Tag  */
  strncpy( dbEntry->tableName, "sngl_block", dbNameLimit );
  strncpy( dbEntry->columnName, "channel", dbNameLimit );

  dbEntry->type           = char_s_ptr;
  dbEntry->numberRows     = params->numberResults;

  dbEntry->rows.chars = (CHAR *) LALMalloc( (strlen(params->channel)+1) * sizeof(CHAR) *params->numberResults);
  if ( !dbEntry->rows.chars )
    {
      ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
    }

  stringLength = strlen(params->channel);
  dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) * params->numberResults );
  if ( !dbEntry->rowDimensions )
    {
      ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
    }

  for (index = 0, current = params->results; (index < params->numberResults); index++, current = ((BlockResult*)(current->next)))
    {
      *(dbEntry->rowDimensions + index) = stringLength;
      strncpy( dbEntry->rows.chars + index * stringLength, params->channel, stringLength * sizeof(CHAR) );
    }

  dbEntry->next = LALCalloc(1, sizeof(dataBase));
  if ( !dbEntry->next )
    {
      ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
    }

  dbEntry->next->previous = dbEntry;
  dbEntry = dbEntry->next;



  /*  Start time  */
  strncpy( dbEntry->tableName, "sngl_block", dbNameLimit);
  strncpy( dbEntry->columnName, "start_time", dbNameLimit );

  dbEntry->type       = int_4s;
  dbEntry->numberRows = params->numberResults;
  dbEntry->rows.int4s = LALMalloc( sizeof(INT4) * params->numberResults);
  if ( !dbEntry->rows.int4s )
    {
      ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
    }

  dbEntry->rowDimensions = NULL;

  for (index = 0, current = params->results; (index < params->numberResults); index++, current = ((BlockResult*)(current->next)))
    { 
      timeZero = dataStart + (((REAL8)(current->startTime))/((REAL8)(params->sampleRate[current->bandLevel])));
      timeSec  = ((INT4)(timeZero));

      *(dbEntry->rows.int4s + index) = timeSec;
    }

  dbEntry->next = LALCalloc(1, sizeof(dataBase));
  if ( !dbEntry->next )
    {
      ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
    }

  dbEntry->next->previous = dbEntry;
  dbEntry = dbEntry->next;



  /*  Start Time Nano Seconds  */
  strncpy( dbEntry->tableName, "sngl_block", dbNameLimit);
  strncpy( dbEntry->columnName, "start_time_ns", dbNameLimit );

  dbEntry->type       = int_4s;
  dbEntry->numberRows = params->numberResults;
  dbEntry->rows.int4s = LALMalloc( sizeof(INT4) * params->numberResults);
  if ( !dbEntry->rows.int4s )
    {
      ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
    }

  dbEntry->rowDimensions = NULL;

  for (index = 0, current = params->results; (index < params->numberResults); index++, current = ((BlockResult*)(current->next)))
    {
      timeZero = dataStart + (((REAL8)(current->startTime))/((REAL8)(params->sampleRate[current->bandLevel])));
      timeSec  = ((INT4)(timeZero));

      timeNan = ( timeZero - timeSec ) * 1000000000.0;
      timeNan = ((INT4)(timeNan));

      *(dbEntry->rows.int4s + index) = timeNan;
    }

  dbEntry->next = LALCalloc(1, sizeof(dataBase));
  if ( !dbEntry->next )
    {
      ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
    }

  dbEntry->next->previous = dbEntry;
  dbEntry = dbEntry->next;



  /*  End time  */
  strncpy( dbEntry->tableName, "sngl_block", dbNameLimit);
  strncpy( dbEntry->columnName, "end_time", dbNameLimit );

  dbEntry->type       = int_4s;
  dbEntry->numberRows = params->numberResults;
  dbEntry->rows.int4s = LALMalloc( sizeof(INT4) * params->numberResults);
  if ( !dbEntry->rows.int4s )
    {
      ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
    }

  dbEntry->rowDimensions = NULL;

  for (index = 0, current = params->results; (index < params->numberResults); index++, current = ((BlockResult*)(current->next)))
    {
      timeZero = dataStart + (((REAL8)(current->startTime + current->duration))/ params->sampleRate[current->bandLevel]);
      timeSec  = ((INT4)(timeZero));

      *(dbEntry->rows.int4s + index) = timeSec;
    }

  dbEntry->next = LALCalloc(1, sizeof(dataBase));
  if ( !dbEntry->next )
    {
      ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
    }

  dbEntry->next->previous = dbEntry;
  dbEntry = dbEntry->next;



  /*  End Time Nanoseconds  */
  strncpy( dbEntry->tableName, "sngl_block", dbNameLimit);
  strncpy( dbEntry->columnName, "end_time_ns", dbNameLimit );

  dbEntry->type       = int_4s;
  dbEntry->numberRows = params->numberResults;
  dbEntry->rows.int4s = LALMalloc( sizeof(INT4) * params->numberResults);
  if ( !dbEntry->rows.int4s )
    {
      ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
    }

  dbEntry->rowDimensions = NULL;

  for (index = 0, current = params->results; (index < params->numberResults); index++, current = ((BlockResult*)(current->next)))
    {
      timeZero = dataStart + (((REAL8)(current->startTime + current->duration))/ params->sampleRate[current->bandLevel]);
      timeSec  = ((INT4)(timeZero));

      timeNan = ( timeZero - timeSec ) * 1000000000.0;
      timeNan = ((INT4)(timeNan));

      *(dbEntry->rows.int4s + index) = timeNan;
    }

  dbEntry->next = LALCalloc(1, sizeof(dataBase));
  if ( !dbEntry->next )
    {
      ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
    }

  dbEntry->next->previous = dbEntry;
  dbEntry = dbEntry->next;



  /*  Band Index Tag  */
  strncpy( dbEntry->tableName, "sngl_block", dbNameLimit);
  strncpy( dbEntry->columnName, "band_index", dbNameLimit );

  dbEntry->type       = int_4s;
  dbEntry->numberRows = params->numberResults;
  dbEntry->rows.int4s = LALMalloc( sizeof(INT4) * params->numberResults);
  if ( !dbEntry->rows.int4s )
    {
      ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
    }

  dbEntry->rowDimensions = NULL;

  for (index = 0, current = params->results; (index < params->numberResults); index++, current = ((BlockResult*)(current->next)))
    {
      *(dbEntry->rows.int4s + index) = current->bandLevel;
    }

  dbEntry->next = LALCalloc(1, sizeof(dataBase));
  if ( !dbEntry->next )
    {
      ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
    }

  dbEntry->next->previous = dbEntry;
  dbEntry = dbEntry->next;



  /*  RhoNStart  */
  strncpy( dbEntry->tableName, "sngl_block", dbNameLimit);
  strncpy( dbEntry->columnName, "rho_n_start", dbNameLimit );

  dbEntry->type       = real_8;
  dbEntry->numberRows = params->numberResults;
  dbEntry->rows.real8 = LALMalloc( sizeof(REAL8) * params->numberResults );
  if ( !dbEntry->rows.real8 )
    {
      ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
    }

  dbEntry->rowDimensions = NULL;

  for (index = 0, current = params->results; (index < params->numberResults); index++, current = ((BlockResult*)(current->next)))
    {
      *(dbEntry->rows.real8 + index) = current->rhoNStart;

      if ( *(dbEntry->rows.real8 + index) > 10000 )
	*(dbEntry->rows.real8 + index) = -7777;
    }

  dbEntry->next = LALCalloc(1, sizeof(dataBase));
  if ( !dbEntry->next )
    {
      ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
    }

  dbEntry->next->previous = dbEntry;
  dbEntry = dbEntry->next;



  /*  RhoNEnd  */
  strncpy( dbEntry->tableName, "sngl_block", dbNameLimit);
  strncpy( dbEntry->columnName, "rho_n_end", dbNameLimit );

  dbEntry->type       = real_8;
  dbEntry->numberRows = params->numberResults;
  dbEntry->rows.real8 = LALMalloc( sizeof(REAL8) * params->numberResults );
  if ( !dbEntry->rows.real8 )
    {
      ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
    }

  dbEntry->rowDimensions = NULL;

  for (index = 0, current = params->results; (index < params->numberResults); index++, current = ((BlockResult*)(current->next)))
    {
      *(dbEntry->rows.real8 + index) = current->rhoNEnd;

      if ( *(dbEntry->rows.real8 + index) > 10000 )
	*(dbEntry->rows.real8 + index) = -7777;
    }

  dbEntry->next = LALCalloc(1, sizeof(dataBase));
  if ( !dbEntry->next )
    {
      ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
    }

  dbEntry->next->previous = dbEntry;
  dbEntry = dbEntry->next;



  /*  Mean Entry  */
  strncpy( dbEntry->tableName, "sngl_block", dbNameLimit);
  strncpy( dbEntry->columnName, "mean", dbNameLimit );

  dbEntry->type       = real_4;
  dbEntry->numberRows = params->numberResults;
  dbEntry->rows.real4 = LALMalloc( sizeof(REAL4) * params->numberResults);
  if ( !dbEntry->rows.real4 )
    {
      ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
    }

  dbEntry->rowDimensions = NULL;

  for (index = 0, current = params->results; (index < params->numberResults); index++, current = ((BlockResult*)(current->next)))
    {
      *(dbEntry->rows.real4 + index) = (REAL4)(current->mean);

      if ( *(dbEntry->rows.real4 + index) > 1000000000 )
	*(dbEntry->rows.real4 + index) = -7777;
    }

  dbEntry->next = LALCalloc(1, sizeof(dataBase));
  if ( !dbEntry->next )
    {
      ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
    }

  dbEntry->next->previous = dbEntry;
  dbEntry = dbEntry->next;



  /*  Variance Entry  */
  strncpy( dbEntry->tableName, "sngl_block", dbNameLimit);
  strncpy( dbEntry->columnName, "variance", dbNameLimit );

  dbEntry->type       = real_4;
  dbEntry->numberRows = params->numberResults;
  dbEntry->rows.real4 = LALMalloc( sizeof(REAL4) * params->numberResults);
  if ( !dbEntry->rows.real4 )
    {
      ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
    }

  dbEntry->rowDimensions = NULL;

  for (index = 0, current = params->results; (index < params->numberResults); index++, current = ((BlockResult*)(current->next)))
    {
      *(dbEntry->rows.real4 + index) = (REAL4)(current->variance);

      if ( *(dbEntry->rows.real4 + index) > 1000000000 )
	*(dbEntry->rows.real4 + index) = -7777;
    }

  dbEntry->next = LALCalloc(1, sizeof(dataBase));
  if ( !dbEntry->next )
    {
      ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
    }

  dbEntry->next->previous = dbEntry;
  dbEntry = dbEntry->next;



  /*  Mean Zero Entry  */
  strncpy( dbEntry->tableName, "sngl_block", dbNameLimit);
  strncpy( dbEntry->columnName, "mean_zero", dbNameLimit );

  dbEntry->type       = real_4;
  dbEntry->numberRows = params->numberResults;
  dbEntry->rows.real4 = LALMalloc( sizeof(REAL4) * params->numberResults);
  if ( !dbEntry->rows.real4 )
    {
      ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
    }

  dbEntry->rowDimensions = NULL;

  for (index = 0, current = params->results; (index < params->numberResults); index++, current = ((BlockResult*)(current->next)))
    {
      *(dbEntry->rows.real4 + index) = (REAL4)(params->meanZero[current->bandLevel]);

      if ( *(dbEntry->rows.real4 + index) > 1000000000 )
	*(dbEntry->rows.real4 + index) = -7777;
    }

  dbEntry->next = LALCalloc(1, sizeof(dataBase));
  if ( !dbEntry->next )
    {
      ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
    }

  dbEntry->next->previous = dbEntry;
  dbEntry = dbEntry->next;



  /*  Variance Zero Entry  */
  strncpy( dbEntry->tableName, "sngl_block", dbNameLimit);
  strncpy( dbEntry->columnName, "variance_zero", dbNameLimit );

  dbEntry->type       = real_4;
  dbEntry->numberRows = params->numberResults;
  dbEntry->rows.real4 = LALMalloc( sizeof(REAL4) * params->numberResults);
  if ( !dbEntry->rows.real4 )
    {
      ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
    }

  dbEntry->rowDimensions = NULL;

  for (index = 0, current = params->results; (index < params->numberResults); index++, current = ((BlockResult*)(current->next)))
    {
      *(dbEntry->rows.real4 + index) = (REAL4)(params->varianceZero[current->bandLevel]);

      if ( *(dbEntry->rows.real4 + index) > 1000000000 )
	*(dbEntry->rows.real4 + index) = -7777;
    }

  dbEntry->next = LALCalloc(1, sizeof(dataBase));
  if ( !dbEntry->next )
    {
      ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
    }

  dbEntry->next->previous = dbEntry;
  dbEntry = dbEntry->next;



  /*  Event Id Tag */
  strncpy( dbEntry->tableName, "sngl_block", dbNameLimit );
  strncpy( dbEntry->columnName, "event_id", dbNameLimit );

  dbEntry->type           = char_s_ptr;
  dbEntry->numberRows     = params->numberResults;

  sprintf(tmp, "sngl_block:event_id:%d",params->eventCounter);

  dbEntry->rows.chars = (CHAR *) LALMalloc( (strlen(tmp)+5) * sizeof(CHAR) *params->numberResults);
  if ( !dbEntry->rows.chars )
    {
      ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
    }

  stringLength = strlen(tmp) + 5;
  dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) * params->numberResults );
  if ( !dbEntry->rowDimensions )
    {
      ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
    }

  current = params->results;

  for (index = 0, current = params->results; (index < params->numberResults); index++, current = ((BlockResult*)(current->next)))
    {
      sprintf(tmp, "sngl_block:event_id:%d",params->eventCounter);
      *(dbEntry->rowDimensions + index) = stringLength;
      strncpy( dbEntry->rows.chars + index * stringLength, tmp, stringLength * sizeof(CHAR) );
      params->eventCounter++;
    }

  current = params->results;

  if (current)
    {
      while( current->next != NULL)
	{
	  tempResult = current;
	  current = ((BlockResult*)(current->next));
	  LALFree(tempResult);
	}

      LALFree(current);
      current = NULL;
      params->results = NULL;
      params->prevResults = NULL;
    }

  dbEntry->next = NULL;

  output->numOutput = 1;

  /*  Allocate Memory  */
  output->result = outputPtr = LALMalloc( (output->numOutput) * sizeof(outPut) );
  if ( !output->result )
    {
      ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
    }

  outputPtr[0].indexNumber	= 0;
  outputPtr[0].search		= experimental;
  outputPtr[0].significant	= 1;	/* 1 - IS significant */
  outputPtr[0].states		= 0;
  outputPtr[0].optional		= 0;
  outputPtr[0].results = params->resultsToStore;

  params->resultsToStore = NULL;

#ifdef BLOCKSEARCH__DATABASE_DEBUG
  printf("events %d \n",params->eventCounter);
#endif

  dbEntry->next = NULL;
  params->lastResultStored = dbEntry;

  RETURN(status);
}
