/******** <lalVerbatim file="BlockTreeCV"> ********
Author: Tibbits, M. M.
$Id: BlockTree.c,v 1.13 2003/07/06 20:06:09 jolien Exp $
********* </lalVerbatim> ********/

/******************************* <lalLaTeX file="BlockTreeC">
\subsection{Module \texttt{BlockTree.c}}
\label{s:BlockTree.c}

Yeah tree algorithm.  WOO

\subsubsection*{Prototypes}
\input{BlockTreeCP}
\index{\texttt{BlockAddQ()}}
\index{\texttt{BlockDelQ()}}
\index{\texttt{BlockSplitLevel()}}

note: BlockAddQ & BlockDelQ are not LALSpec functions.

\subsection*{Description}

\subsection*{Algorithms}

\subsection*{Uses}

\subsection*{Notes}

\vfill{\footnotesize\input{BlockTreeCV}}

 *************************************************** </lalLaTeX> */

#include "Block.h"
#include "lal/LALMoment.h"

NRCSID( BLOCKTREEC, "$Id: BlockTree.c,v 1.13 2003/07/06 20:06:09 jolien Exp $");

/*
 * Function BlockAddQ
 *
 * Adds the pointer newSegment to the tail of the Queue.
 * Not LALSpec Function  ( No status )
 */

/************************ <lalVerbatim file="BlockTreeCP"> */
void BlockAddQ
(
	LALStatus	*status,
	bandSegment	*newSegment,
	void		*searchParams
)
/*********************************************** </lalVerbatim> */
{
	/*  Variable Declarations  */
	BlockSearchParams	*params;

	INITSTATUS( status, "BlockTree", BLOCKTREEC );

	/*  Inititalize Variables  */
	params = ((BlockSearchParams*)(searchParams));

	newSegment->next = NULL;

	if( params->tail == NULL )
	{
		params->head = newSegment;
	}
	else
	{
		((bandSegment*)(params->tail->next)) = newSegment;
	}

	params->tail = newSegment;

	RETURN(status);
}


/*
 * Function BlockDelQ
 *
 * Returns pointer to head segment and removes head segment from the
 * Queue.  If head is NULL, function returns NULL.
 * Not LALSpec Function  ( No status )
 */

/************************ <lalVerbatim file="BlockTreeCP"> */
bandSegment*	BlockDelQ
(
	void		*searchParams
)
/*********************************************** </lalVerbatim> */
{
	/*  Variable Declarations  */
	BlockSearchParams	*params;
	bandSegment		*tmp;

	/*  Inititalize Variables  */
	params = ((BlockSearchParams*)(searchParams));

	if( params->head == NULL )
	{
		return( NULL );
	}

	tmp = params->head;

	if( params->head == params->tail )
	{
		params->head = params->tail = NULL;
	}
	else
	{
	  params->head = ((bandSegment*)(params->head->next));
	}

	return( tmp );
}


/*
 * Function BlockSplitLevel
 *
 * Parse to split input segment into two separate pieces, left & right halves.
 * A segment will not be created if it would be smaller than
 * params->minVectorLength.  Input will not be split if rho2 isn't higher than
 * the threshold.
 *
 */

/************************ <lalVerbatim file="BlockTreeCP"> */
void BlockSplitLevel
(
	LALStatus	*status,
	bandSegment	*input,
	void		*searchParams
)
/*********************************************** </lalVerbatim> */
{
	/*  Variable Declarations  */
	BlockSearchParams	*params;
	bandSegment		*tmp;
	BlockResult		*current;
	UINT4			iterator;

	INITSTATUS( status, "BlockTree", BLOCKTREEC );
	ATTATCHSTATUSPTR (status);

	/*  Inititalize Variables  */
	params	= ((BlockSearchParams*)(searchParams));
	input->left = input->right = NULL;
	tmp = NULL;

	/*  Need to Scale the Threshold  */
	input->rhoN = input->rhoN + log( params->scaleFactor * (pow(input->data->length,params->scaleExponent)));

	/*  if we aren't above threshold, we're done.  */
	if( input->rhoN < log(params->threshold[input->level]) )
	{
		DETATCHSTATUSPTR( status );
		RETURN(status);
	}

	/*  Since we're above threshold, add current segment to slave result chain  */
	current = (BlockResult*)(LALMalloc ( sizeof(BlockResult)));
	if( !current)
	{
		ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
	}

	current->bandLevel = input->level;

	/* We need to do something about segmentID */
	/*  current->segmentID;  */

	current->startTime = input->startTime + input->index;
	current->duration  = input->data->length - input->index;;
	current->value     = input->value;
	current->index     = input->index;
	current->rhoNStart = input->rhoN;

	current->next = NULL;

	if( params->results == NULL )
	{
		params->results = current;
	}
	else
	{
		((bandSegment*)(params->prevResults->next)) = current;
	}

	params->prevResults = current;

	params->numberResults++;

	current = NULL;

	/*  Chop Vector based on Input Parameters  */

	if( input->index > params->minVectorLength )
	{
		tmp = (bandSegment*)(LALMalloc( sizeof( bandSegment)));
		if( !tmp )
		{
			ABORT(status, BLOCKH_EDATA, BLOCKH_MSGEDATA);
		}

		((bandSegment*)(input->left)) = tmp;
		
		/*  Initialize new Segment  */
		tmp->rhoN  = tmp->index = tmp->value = 0;
		tmp->left  = NULL;
		tmp->right = NULL;
		tmp->next  = NULL;
		tmp->data  = NULL;

		/*  Copy Id information  */
		tmp->level = input->level;
		tmp->startTime = input->startTime;

		/*  Calculate Duration  */
		tmp->duration = 0;

		/*  Copy Data  */
		LALDCreateVector( status->statusPtr, &(tmp->data), input->index);
		CHECKSTATUSPTR( status );

		for( iterator = 0; iterator < input->index; iterator++ )
		{
			tmp->data->data[iterator] = input->data->data[iterator];
		}
	}

	if( (input->data->length - input->index) > params->minVectorLength )
	{
		tmp = (bandSegment*)(LALMalloc( sizeof( bandSegment)));
		if( !tmp )
		{
			ABORT(status, BLOCKH_EDATA, BLOCKH_MSGEDATA);
		}

		((bandSegment*)(input->right)) = tmp;

		/*  Initialize new Segment  */
		tmp->rhoN = tmp->index = tmp->value = 0;
                tmp->left  = NULL;
		tmp->right = NULL;
		tmp->next  = NULL;
		tmp->data  = NULL;

		/*  Copy Id information  */
		tmp->level = input->level;

		/*  Calculate startTime  */
		tmp->startTime = input->index + input->startTime;

		/*  Calculate Duration  */
		tmp->duration = 0;

		/*  Copy Data  */
		LALDCreateVector( status->statusPtr, &(tmp->data), input->data->length - input->index);
		CHECKSTATUSPTR( status );

		for( iterator = input->index; iterator < input->data->length; iterator++ )
		{
			tmp->data->data[iterator - input->index] = input->data->data[iterator];
		}
	}

	DETATCHSTATUSPTR(status);
	RETURN(status);
}
