/**** <lalVerbatim file="InitSearchHV"> ***********
Author: Julien Sylvestre
$Id: InitSearch.h,v 1.2 2002/11/18 23:23:32 julien Exp $ 
***** </lalVerbatim> ***********************************/

#ifndef _INITSEARCH_H
#define _INITSEARCH_H

#include <LALWrapperInterface.h>
#include <lal/LALDatatypes.h>
#include <lal/LALRCSID.h>
#include <lal/CoherentEstimation.h>
#include <lal/TFClusters.h>

#ifdef  __cplusplus
extern "C" {
#endif

NRCSID (INITSEARCHH, "$Id: InitSearch.h,v 1.2 2002/11/18 23:23:32 julien Exp $");

  typedef enum {SNGL_BURST, GDS_TRIGGER} DbTable;

  typedef enum {LHO2K,LHO4K,LLO4K,VIRGO,TAMA,GEO} COHIFO;

  typedef struct tagCohereSearchParams {


  /**** General Parameters ****/

  UINT4 SearchMethod; /* search method */

  BOOLEAN searchMaster; /* 1 if searchMaster */

  UINT4 NIFO; /* number of IFOs in network */

  COHIFO *IFOIND; /* ordered list of IFO names */

  DetectorsData IFOData; /* time series from each IFO */

  CoherentEstimation IFOInfo; /* information from all IFOs */

  /* equatorial celestial coordinates search: */
  REAL8 AlphaMin, AlphaMax; /* bounds of alpha search interval */

  UINT4 NAlpha; /* will search NAlpha points in [AlphaMin, AlphaMax] */

  REAL8 DeltaMin, DeltaMax; /* bounds of Delta search interval */

  UINT4 NDelta; /* will search NDelta points in [DeltaMin, DeltaMax] */

  /* polarization angle search: */
  REAL8 PsiMin, PsiMax; /* bounds of Psi search interval */

  UINT4 NPsi; /* will search NPsi points in [PsiMin, PsiMax] */

    /* search over angle between + and x (take sin of) */
    REAL8 plusDotcrossMin, plusDotcrossMax;
    UINT4 NplusDotcross;

    /* search over ratio +/x */
    REAL8 plus2crossMin, plus2crossMax;
    UINT4 Nplus2cross;

  REAL8 *Alpha, *Delta, *Psi, *plusDotcross, *plus2cross; /* vectors of trial points */

  UINT4 numSlaves; /* number of slaves */

  /**** TFCLUSTERS parameters ****/
 
  UINT4 NData; /* total number of data to be searched */

  REAL8 T; /* time resolution */

  UINT4 fs; /* sampling frequency of channel */

  REAL4 bpp; /* black pixel probability */

  REAL4 eGoal; /* absolute error goal on power threshld */

  REAL4 Nsigma; /* number of std above mean to reject in second pass at estimation of mean and variance (Q and P0) */

  CListDir* dir;            /* container for the instructions */

  BOOLEAN WriteBlobs; /* set to 1 if pixel coordinates are to saved in sngl_transdata */

  DbTable table; /* database table name */

  char name[256]; /* channel name */

    UINT4 cminf, cmaxf, cmint, cmaxt; /* tf-plane limits for interesting cluster */

    REAL8 cmintR8, cmaxtR8, cbw, ccf;

} CohereSearchParams;


/******** <lalErrTable file="COHInitSearchHErrTab"> ********/
#define INITSEARCHH_ENULL     1
#define INITSEARCHH_ENNUL     2
#define INITSEARCHH_EALOC     3
#define INITSEARCHH_EARGS     4
#define INITSEARCHH_ESPOS      5
#define INITSEARCHH_ENAN      6
#define INITSEARCHH_EAOR      7
#define INITSEARCHH_EPOS      8
#define INITSEARCHH_EMMF      9
#define INITSEARCHH_ENAM      10
#define INITSEARCHH_ETAB      11
#define INITSEARCHH_EIFO      12
#define INITSEARCHH_E02PI     13

#define INITSEARCHH_MSGENULL     "Null pointer"
#define INITSEARCHH_MSGENNUL     "Non-null pointer"
#define INITSEARCHH_MSGEALOC     "Memory allocation error"
#define INITSEARCHH_MSGEARGS     "Wrong number of arguments"
#define INITSEARCHH_MSGESPOS      "Parameter <= 0 expected to be strictly positive"
#define INITSEARCHH_MSGENAN       "Numerical parameter is NaN"
#define INITSEARCHH_MSGEAOR      "Parameter expected to be in [0,1] is out of range"
#define INITSEARCHH_MSGEPOS      "Parameter < 0 when expected to be positive or zero"
#define INITSEARCHH_MSGEMMF      "min parameter > max parameter"
#define INITSEARCHH_MSGENAM       "Channel/IFO name is zero length"
#define INITSEARCHH_MSGETAB       "Unknown table name"
#define INITSEARCHH_MSGEIFO       "Unsupported IFO name"
#define INITSEARCHH_MSGE02PI      "Parameter should be in [0, 2 Pi]"
/******** </lalErrTable> ********/


#ifdef  __cplusplus
}
#endif

#endif /* _INITSEARCH_H  */


