/*----------------------------------------------------------------------- 
 * 
 * File Name: InspiralExch.c
 *
 * Author: Allen, B., Brown, D. A. and Creighton, J. D. E.
 * 
 * Revision: $Id: InspiralExch.c,v 1.2 2003/06/09 05:06:26 duncan Exp $
 * 
 *-----------------------------------------------------------------------
 */

#if 0
<lalVerbatim file="InspiralExchCV">
Author: Allen, B., Brown, D. A. and Creighton, J. D. E.
$Id: InspiralExch.c,v 1.2 2003/06/09 05:06:26 duncan Exp $
</lalVerbatim>

<lalLaTeX>
\subsection{Module \texttt{InspiralExch.c}}
\label{ss:InspiralExch.c}

This module functions to exchange \texttt{findchirp} specific data types
using MPI.

\subsubsection*{Prototypes}
\vspace{0.1in}
\input{InspiralExchCP}
\idx{ExchangeInspiralTemplate()}
\idx{ExchangeInspiralEventList()}
\idx{ExchangeTemplateBank()}

\subsubsection*{Description}


\subsubsection*{Algorithm}

None.

\subsubsection*{Uses}
\begin{verbatim}
LALCalloc()
LALFree()
\end{verbatim}

\subsubsection*{Notes}

\vfill{\footnotesize\input{InspiralExchCV}}
</lalLaTeX>
#endif

#include <lal/LALStdlib.h>
#include <lal/DataBuffer.h>
#include <lal/FindChirpEngine.h>
#include "InspiralExch.h"

NRCSID (INSPIRALEXCHC, "$Id: InspiralExch.c,v 1.2 2003/06/09 05:06:26 duncan Exp $");

/* <lalVerbatim file="InspiralExchCP"> */
void
ExchangeInspiralTemplate (
    LALStatus          *status,
    InspiralTemplate   *tmplt,
    ExchParams         *exchParams
    )
/* </lalVerbatim> */
{
  CHARVector box; /* a box to hold some bytes of data */

  INITSTATUS (status, "ExchangeInspiralTemplate", INSPIRALEXCHC);
  ATTATCHSTATUSPTR (status);

  /* only do a minimal check to see if arguments are somewhat reasonable */
  ASSERT (exchParams, status, INSPIRALEXCHH_ENULL, INSPIRALEXCHH_MSGENULL);
  ASSERT (tmplt, status, INSPIRALEXCHH_ENULL, INSPIRALEXCHH_MSGENULL);

  /* stuff the template into a box */
  box.length = sizeof (InspiralTemplate);
  box.data   = (CHAR *) tmplt;

  if (exchParams->send)  /* I am sending */
  {
    INT4 dest = exchParams->partnerProcNum;

    /* send box */
    LALMPISendCHARVector (status->statusPtr, &box, dest, exchParams->mpiComm);
    CHECKSTATUSPTR (status);

    /* exchange the length of the segmentId vector */
    LALExchangeUINT4( status->statusPtr, &(tmplt->segmentIdVec->length),
        exchParams );
    CHECKSTATUSPTR (status);

    /* and then send the vector */
    LALMPISendINT4Vector( status->statusPtr, tmplt->segmentIdVec,
        dest, exchParams->mpiComm);
    CHECKSTATUSPTR (status);
  }
  else /* I am receiving */
  {
    INT4 source = exchParams->partnerProcNum;
    UINT4      length;

    /* receive box */
    LALMPIRecvCHARVector (status->statusPtr, &box, source, exchParams->mpiComm);
    CHECKSTATUSPTR (status);

    /* exchange the length of the segmentId vector */
    LALExchangeUINT4( status->statusPtr, &length, exchParams );
    CHECKSTATUSPTR (status);
    
    /* allocate the memory for it */
    tmplt->segmentIdVec = NULL;
    LALI4CreateVector( status->statusPtr, &(tmplt->segmentIdVec), length);
    CHECKSTATUSPTR( status );

    /* and then recv the vector */
    LALMPIRecvINT4Vector( status->statusPtr, tmplt->segmentIdVec,
        source, exchParams->mpiComm);
    CHECKSTATUSPTR (status);
    
  }

  DETATCHSTATUSPTR (status);
  RETURN (status);
}


/* <lalVerbatim file="InspiralExchCP"> */
void
ExchangeInspiralEvent (
    LALStatus          *status,
    SnglInspiralTable  *event,
    ExchParams         *exchParams
    )
/* </lalVerbatim> */
{
  CHARVector box; /* a box to hold some bytes of data */

  INITSTATUS (status, "ExchangeInspiralEvent", INSPIRALEXCHC);
  ATTATCHSTATUSPTR (status);

  /* only do a minimal check to see if arguments are somewhat reasonable */
  ASSERT (exchParams, status, INSPIRALEXCHH_ENULL, INSPIRALEXCHH_MSGENULL);
  ASSERT (event, status, INSPIRALEXCHH_ENULL, INSPIRALEXCHH_MSGENULL);

  /* stuff the event into a box */
  box.length = sizeof (SnglInspiralTable);
  box.data   = (CHAR *) event;

  if (exchParams->send)  /* I am sending */
  {
    INT4 dest = exchParams->partnerProcNum;

    /* send box */
    LALMPISendCHARVector (status->statusPtr, &box, dest, exchParams->mpiComm);
    CHECKSTATUSPTR (status);
  }
  else /* I am receiving */
  {
    INT4 source = exchParams->partnerProcNum;

    /* receive box */
    LALMPIRecvCHARVector (status->statusPtr, &box, source, exchParams->mpiComm);
    CHECKSTATUSPTR (status);
  }

  DETATCHSTATUSPTR (status);
  RETURN (status);
}


/* <lalVerbatim file="InspiralExchCP"> */
void
ExchangeInspiralEventList (
    LALStatus          *status,
    SnglInspiralTable **eventHead,
    ExchParams         *exchParams
    )
/* </lalVerbatim> */
{
  SnglInspiralTable    *eventCurrent = NULL;
  SnglInspiralTable    *prev         = NULL;
  SnglInspiralTable    *events       = NULL;
  UINT4                 numEvents    = 0;
  INT4                  i, code;
  MPI_Status            stat;

  INITSTATUS (status, "ExchangeInspiralEvent", INSPIRALEXCHC);
  ATTATCHSTATUSPTR (status);

  if (exchParams->send)  /* I am sending */
  {

    /* check that we have a bank to send */
    ASSERT (*eventHead, status, INSPIRALEXCHH_ENULL, INSPIRALEXCHH_MSGENULL);

    /* count the events */
    for (eventCurrent = *eventHead;
        eventCurrent != NULL;
        eventCurrent = eventCurrent->next
        )
    {
      numEvents++;
    }

    /* exchange the number of events */
    LALExchangeUINT4( status->statusPtr, &numEvents, exchParams );
    CHECKSTATUSPTR (status);

    /* allocate memory for array of pointers to events */
    events = NULL;
    events = (SnglInspiralTable *) 
      LALCalloc (numEvents , sizeof(SnglInspiralTable));

    /*  Make sure that the allocation was succesful */
    if ( ! events )
    {
      ABORT (status, INSPIRALEXCHH_ENULL, INSPIRALEXCHH_MSGENULL);
    }

    /* copy out events into array */
    i=0;
    for (eventCurrent = *eventHead;
        eventCurrent != NULL;
        eventCurrent = eventCurrent->next
        )
    {
      i++;
      memcpy( (events + i-1), eventCurrent, sizeof(SnglInspiralTable));
    }

    /* send the array */
    code = MPI_Send( events, numEvents * sizeof(SnglInspiralTable), MPI_BYTE, 
        exchParams->partnerProcNum, MPIZ, exchParams->mpiComm );
    if ( code != MPI_SUCCESS )
    {
      ABORT( status, INSPIRALEXCHH_EMPIE, INSPIRALEXCHH_MSGEMPIE);
    }

    /* free the memory */
    LALFree(events);
  }
  else /* I am receiving */
  {

    /* check that this is a new list */
    ASSERT (!*eventHead, status, INSPIRALEXCHH_ENNUL, INSPIRALEXCHH_MSGENNUL);

    /* receive the number of events */
    LALExchangeUINT4( status->statusPtr, &numEvents, exchParams );
    CHECKSTATUSPTR (status);

    /* allocate memory for array of pointers to events */
    events = NULL;
    events = (SnglInspiralTable *) 
      LALCalloc (numEvents, sizeof(SnglInspiralTable));

    /*  Make sure that the allocation was succesful */
    if ( ! events )
    {
      ABORT (status, INSPIRALEXCHH_ENULL, INSPIRALEXCHH_MSGENULL);
    }

    code = MPI_Recv( events, numEvents * sizeof(SnglInspiralTable), MPI_BYTE, 
        exchParams->partnerProcNum, MPIZ, exchParams->mpiComm, &stat );
    if ( code != MPI_SUCCESS || stat.MPI_ERROR != MPI_SUCCESS )
    {
      ABORT( status, INSPIRALEXCHH_EMPIE, INSPIRALEXCHH_MSGEMPIE);
    }

    /* recieve the template bank */
    i=0;
    do
    {
      /* create memory for the template */
      eventCurrent = (SnglInspiralTable *) 
        LALCalloc ( 1, sizeof(SnglInspiralTable) );

      /* make a note of the first node in the list to return */
      if ( *eventHead == NULL ) *eventHead = eventCurrent;

      /* copy the event from the array */
      memcpy( eventCurrent, (events + i), sizeof(SnglInspiralTable));
 
      /* point the previous node to this node */
      if ( prev != NULL ) prev->next = eventCurrent;
      prev = eventCurrent;

      /* increment counter */
      ++i;
    } while ( eventCurrent->next != NULL && i< (INT4)numEvents);

    /* Terminate the linked list */
    eventCurrent->next = NULL;

    LALFree(events);
  }

  DETATCHSTATUSPTR (status);
  RETURN (status);
}


/* <lalVerbatim file="InspiralExchCP"> */
void
ExchangeTemplateBank (
    LALStatus          *status,
    InspiralTemplate  **tmpltHead,
    ExchParams          *exchParms
    )
/* </lalVerbatim> */
{
  InspiralTemplate   *tmpltCurrent = NULL;
  InspiralTemplate   *tmpltFine    = NULL;
  InspiralTemplate   *prev         = NULL;

  INITSTATUS (status, "ExchangeTemplateBank", INSPIRALEXCHC);
  ATTATCHSTATUSPTR (status);

  if (exchParms->send)  /* I am sending */
  {

    /* check that we have a bank to send */
    ASSERT (*tmpltHead, status, INSPIRALEXCHH_ENULL, INSPIRALEXCHH_MSGENULL);

    /* exchange the template bank */
    for (tmpltCurrent = *tmpltHead;
        tmpltCurrent != NULL;
        tmpltCurrent = tmpltCurrent->next
        )
    {
      ExchangeInspiralTemplate (status->statusPtr, tmpltCurrent, exchParms);
      CHECKSTATUSPTR (status);

      /* exchange the fine grids */
      if (tmpltCurrent->fine != NULL)
      {
        ExchangeTemplateBank (status->statusPtr, 
            &(tmpltCurrent->fine), exchParms);
        CHECKSTATUSPTR (status);
      }
    }

    ASSERT (!tmpltCurrent, status, 
        INSPIRALEXCHH_ENNUL, INSPIRALEXCHH_MSGENNUL);

  }
  else /* I am receiving */
  {

    /* check that this is a new list */
    ASSERT (!*tmpltHead, status, INSPIRALEXCHH_ENNUL, INSPIRALEXCHH_MSGENNUL);

    /* recieve the template bank */
    do
    {
      /* create memory for the template */
      tmpltCurrent = (InspiralTemplate *) 
        LALCalloc ( 1, sizeof(InspiralTemplate) );

      /* make a note of the first node in the list to return */
      if ( *tmpltHead == NULL ) *tmpltHead = tmpltCurrent;

      /* recieve the template */
      ExchangeInspiralTemplate (status->statusPtr, tmpltCurrent, exchParms);
      CHECKSTATUSPTR (status);

      /* get the fine grids */
      if (tmpltCurrent->fine != NULL)
      {
        tmpltFine = NULL;
        ExchangeTemplateBank (status->statusPtr, &tmpltFine, exchParms);
        CHECKSTATUSPTR (status);

        /* set the fine pointer */
        tmpltCurrent->fine = tmpltFine;
      }

      /* point the previous node to this node */
      if ( prev != NULL ) prev->next = tmpltCurrent;
      prev = tmpltCurrent;

    } while ( tmpltCurrent->next != NULL );

  }

  DETATCHSTATUSPTR (status);
  RETURN (status);
}
