/*************** <lalVerbatim file="ConditionCV"> ****************
Author: Klimenko, S. and Yakushin, I.
$Id: Condition.c,v 1.3 2003/05/29 20:47:53 igor Exp $
**************** </lalVerbatim> *************************************/
#include <stdio.h>
#include <string.h>
#include <lal/LALStdlib.h>
#include <lal/Units.h>
#include <Wave.h>
#include <BuildDB.h>


NRCSID(CONDITIONC, "$Id" );


void LALConditionData(LALStatus *status,
		      LALSearchInput *inout,
		      void *searchParams,
		      LALMPIParams *mpiParams
		      )
{
  WaveParams *params;

  INT4 oneTIME=-1, twoTIME=-1, oneFREQ=-1, twoFREQ=-1, i;

  INITSTATUS( status, "LALConditionData", CONDITIONC );
  ATTATCHSTATUSPTR( status );

  mpiParams=NULL;

  params=(WaveParams*)searchParams;

  for(i=0;i<inout->numberSequences;i++)
    {
      if(strncmp(inout->sequences[i].comment,"one_time",8)==0)
	{
	  oneTIME=i;
	}
      if(strncmp(inout->sequences[i].comment,"two_time",8)==0)
	{
	  twoTIME=i;
	}
      if(strncmp(inout->sequences[i].comment,"one_freq",8)==0)
	{
	  oneFREQ=i;
	}
      if(strncmp(inout->sequences[i].comment,"two_freq",8)==0)
	{
	  twoFREQ=i;
	}
    }



  /* Checking data consistency */

  if(!params->noise_rms_flag && inout->numberSequences<2)
    {
      ABORT(status,WAVEH_ENSEQ,WAVEH_MSGENSEQ);
    }
  if(params->noise_rms_flag && inout->numberSequences<4)
    {
      ABORT(status,WAVEH_ENSEQ,WAVEH_MSGENSEQ);
    }

  if(inout->sequences[oneTIME].type!=real_4 ||
     inout->sequences[twoTIME].type!=real_4)
    {
      ABORT(status,WAVEH_ETYPES,WAVEH_MSGETYPES);
    }

  if(params->noise_rms_flag &&
     (inout->sequences[oneFREQ].type!=real_4 || 
      inout->sequences[twoFREQ].type!=real_4))
    {
      ABORT(status,WAVEH_ETYPES,WAVEH_MSGETYPES);
    }

  if(inout->sequences[oneTIME].space!=timeD ||
     inout->sequences[twoTIME].space!=timeD)
    {
      ABORT(status,WAVEH_ESPACES,WAVEH_MSGESPACES);
    }

  if(params->noise_rms_flag && 
     (inout->sequences[oneFREQ].space!=freqD ||
     inout->sequences[twoFREQ].space!=freqD))
    {
      ABORT(status,WAVEH_ESPACES,WAVEH_MSGESPACES);
    }

  if(inout->sequences[oneTIME].range.dTime.numberSamples!=
     inout->sequences[twoTIME].range.dTime.numberSamples ||
     inout->sequences[oneTIME].numberDimensions!=
     inout->sequences[twoTIME].numberDimensions || 
     inout->sequences[oneTIME].dimensions[0]!=
     inout->sequences[twoTIME].dimensions[0])
    {
      ABORT(status,WAVEH_EDIMS,WAVEH_MSGEDIMS);
    }

  if(params->noise_rms_flag && 
     (inout->sequences[oneFREQ].range.dFreq.numberSamples!=
      inout->sequences[twoFREQ].range.dFreq.numberSamples ||
      inout->sequences[oneFREQ].numberDimensions!=
      inout->sequences[twoFREQ].numberDimensions ||
      inout->sequences[oneFREQ].dimensions[0]!=
      inout->sequences[twoFREQ].dimensions[0]
      )
     )
    {
      ABORT(status,WAVEH_EDIMS,WAVEH_MSGEDIMS);
    }


  /* Getting input data */

  params->start.gpsSeconds=
    inout->sequences[oneTIME].range.dTime.startSec;
  params->start.gpsNanoSeconds=
    inout->sequences[oneTIME].range.dTime.startNan;
  params->stop.gpsSeconds=
    inout->sequences[oneTIME].range.dTime.stopSec;
  params->stop.gpsNanoSeconds=
    inout->sequences[oneTIME].range.dTime.stopNan;

  params->oneIn->data->epoch.gpsSeconds=
    inout->sequences[oneTIME].range.dTime.startSec;
  params->oneIn->data->epoch.gpsNanoSeconds=
    inout->sequences[oneTIME].range.dTime.startNan;

  params->twoIn->data->epoch.gpsSeconds=
    inout->sequences[twoTIME].range.dTime.startSec;
  params->twoIn->data->epoch.gpsNanoSeconds=
    inout->sequences[twoTIME].range.dTime.startNan;

  strcpy(params->oneIn->data->name, params->nameOne);
  strcpy(params->twoIn->data->name, params->nameTwo);

  params->oneIn->data->data->data=
    inout->sequences[oneTIME].data.real4;
  params->oneIn->data->data->length=
    inout->sequences[oneTIME].range.dTime.numberSamples;
  params->oneIn->data->deltaT=
    inout->sequences[oneTIME].range.dTime.timeStepSize;

  params->twoIn->data->data->data=
    inout->sequences[twoTIME].data.real4;
  params->twoIn->data->data->length=
    inout->sequences[twoTIME].range.dTime.numberSamples;
  params->twoIn->data->deltaT=
    inout->sequences[twoTIME].range.dTime.timeStepSize;


  if(params->noise_rms_flag)
    {
      params->onePSD->epoch.gpsSeconds=
	inout->sequences[oneFREQ].range.dFreq.gpsStartTimeSec;
      params->onePSD->epoch.gpsNanoSeconds=
	inout->sequences[oneFREQ].range.dFreq.gpsStartTimeNan;
      params->onePSD->f0=
	inout->sequences[oneFREQ].range.dFreq.baseFreq;
      
      params->twoPSD->epoch.gpsSeconds=
	inout->sequences[twoFREQ].range.dFreq.gpsStartTimeSec;
      params->twoPSD->epoch.gpsNanoSeconds=
	inout->sequences[twoFREQ].range.dFreq.gpsStartTimeNan;
      params->twoPSD->f0=
	inout->sequences[twoFREQ].range.dFreq.baseFreq;      

      strcpy(params->onePSD->name, params->nameOne);
      strcpy(params->twoPSD->name, params->nameTwo);
      
      params->onePSD->data->data=
	inout->sequences[oneFREQ].data.real4;
      params->onePSD->data->length=
	inout->sequences[oneFREQ].range.dFreq.numberSamples;
      params->onePSD->deltaF=
	inout->sequences[oneFREQ].range.dFreq.freqStepSize;

      params->twoPSD->data->data=
        inout->sequences[twoFREQ].data.real4;
      params->twoPSD->data->length=
	inout->sequences[twoFREQ].range.dFreq.numberSamples;
      params->twoPSD->deltaF=
	inout->sequences[twoFREQ].range.dFreq.freqStepSize;

      /*
      printf("oneTIME=%d, oneFREQ=%d, twoTIME=%d, twoFREQ=%d\n", oneTIME, oneFREQ, twoTIME, twoFREQ);
      printf("In condition: onePSD[0]=%g onePSD[1]=%g\n",(float)(inout->sequences[oneFREQ].data.real4[0]), 
	     (float)(inout->sequences[oneFREQ].data.real4[1]));
      printf("In condition: length=%d, deltaF=%f, startSec=%d, startNan=%d, f0=%f, name=%s, comment=%s, type=%d, space=%d\n",
	     (int)(inout->sequences[oneFREQ].range.dFreq.numberSamples),
	     (float)(inout->sequences[oneFREQ].range.dFreq.freqStepSize),
	     (int)(inout->sequences[oneFREQ].range.dFreq.gpsStartTimeSec),
	     (int)(inout->sequences[oneFREQ].range.dFreq.gpsStartTimeNan),
	     (float)(inout->sequences[oneFREQ].range.dFreq.baseFreq),
	     inout->sequences[oneFREQ].name,
	     inout->sequences[oneFREQ].comment,
	     (int)(inout->sequences[oneFREQ].type), 
	     (int)(inout->sequences[oneFREQ].space)
	     );
      */

  }


  
  DETATCHSTATUSPTR( status );
  RETURN( status );
}

